--[[
Copyright 2010, 2011 William Matthews. All rights reserved.

See LICENSE for license terms.
]]

local CB = select(2,...)
local P = CB.Private
local Hooks = P:NewModule("Hooks", "AceEvent-3.0")
local L = P.L

local function hookTSM()
	if not TSMAPI or not TSMAPI.AddPriceSource then
		P:DebugPrint("TSM hook failed")
		return
	end

	TSMAPI:AddPriceSource(
	   "CostBasis",L["Cost Basis"],
	   function(itemLink,itemID) 
	      itemID = itemID or itemLink -- Depending on the version of TSM, there may be one or two args
	      local rv=CB:QueryItem(itemID)
	      -- P:DebugPrint({"TSM CB QUERY", itemLink, itemID, rv})
	      return rv
	   end)
	P:DebugPrint("TSM hooked")
end

local function hookAucAdvanced()
	if not AucAdvanced then return end
	
	local libType, libName = "Stat", "CostBasis"
	local lib,parent,private = AucAdvanced.NewModule(libType, libName)
	if not lib then return end
	local print,decode,_,_,replicate,empty,get,set,default,debugPrint,fill = AucAdvanced.GetModuleLocals()
		
	function lib.IsValidAlgorithm()
		return true
	end

	function lib.CanSupplyMarket()
		return true
	end
	
	
	function lib.GetPrice(hyperlink,serverKey)
		local linkType,itemID = decode(hyperlink)
		if linkType=="item" and itemID and itemID > 0 then
			return CB:QueryItem(itemID)
		end
	end
	
	function lib.GetPriceColumns()
		return L["Cost Basis"]
	end
	
	local array={}
	function lib.GetPriceArray(hyperlink,serverKey)
		array.price=lib.GetPrice(hyperlink,serverKey)
		return array
	end

	P:DebugPrint("AucAdvanced hooked")
end

local function onError(err)
	if not P.db or P.db.global.debug then
		geterrorhandler()(err)
	end
end

-- Note: This is OnEnable not OnInitialize so that this runs after other addons are loaded
function Hooks:OnEnable()
	xpcall(hookTSM,onError)
	xpcall(hookAucAdvanced,onError)
end
