--[[
Copyright 2010, 2011 William Matthews. All rights reserved.

See LICENSE for license terms.
]]

local CB = select(2,...)
local P = CB.Private
local Tooltip = P:NewModule("Tooltip", "AceEvent-3.0")

local UNCRAFTING = {
	{skill = "Milling", exp = function(i) return P.getExpectedMilling(i) end, count=5 },
	{skill = "Disenchant", exp = function(i) return P.getExpectedDisenchant(i) end, count=1 },
	{skill = "Prospecting", exp = function(i) return P.getExpectedProspecting(i) end, count=5 },
}

local function DoTooltip(tooltip,link,ahPrice)
--	DebugPrint({tooltip=tooltip,link=link})
	if not P.G.tooltip then return end
	if not link or not tooltip then return end
	local id = P.idOfItem(link)
	local cb = CB:QueryItem(link,1)
	local num = P:GetTotalCBItemCount(id)
	tooltip:AddLine(" ")
	tooltip:AddLine(format("Cost basis (x|cffffffff%d|r): |cffffffff%s|r",num,P.moneyOrUnknown(cb)))
	if P.G.debug then
		tooltip:AddLine(format("  id: |cffffffff%d|r",id))
	end
	
	local color = "ffffffff"
	if ahPrice then
		color = "ffaaaaff"
		tooltip:AddLine(string.format("  AH Price: |c%s%s|r",color,P.moneyString(ahPrice)))
		cb=ahPrice
	end
	-- TODO: Make this a preference, default to off.
	--[[
	for _, unc in pairs(UNCRAFTING) do
		local exp = unc.exp(id)
		if exp then
			local relValue = P.relativeValues(exp)
			tooltip:AddLine(unc.skill..":")
			for mat, count in pairs(exp) do
				tooltip:AddLine(format("  %s x|cffffffff%.2g|r at a CB of |c%s%s|r (currently |cffffffff%s|r)",
				  -- relValue[mat] fraction of the cost goes to this mat
					P.coloredNameOfItem(mat), count/unc.count, color,P.moneyOrUnknown(cb and cb * relValue[mat] * unc.count / count), P.moneyOrUnknown(CB:QueryItem(mat)) ))
			end
			
		end
	end
	]]
end

local gameTooltipDone = false
local itemTooltipDone = false
local ahPrice, ahId

local function Hook_GameTooltip_OnTooltipSetItem(tooltip, ...)
	if not gameTooltipDone and tooltip then
		local _, link = tooltip:GetItem()
		gameTooltipDone = true
		if link then
			DoTooltip(tooltip,link,P.idOfItem(link)==ahId and ahPrice)
		end
	end
end

local function Hook_GameTooltip_OnTooltipCleared(tooltip, ...)
	gameTooltipDone = false
	ahPrice = nil
	ahId = nil
end

local function Hook_ItemRefTooltip_OnTooltipSetItem(tooltip, ...)
	if not itemTooltipDone and tooltip then
		local _, link = tooltip:GetItem()
		itemTooltipDone = true
		if link then
			DoTooltip(tooltip,link)
		end
	end
end

local function Hook_ItemRefTooltip_OnTooltipCleared(tooltip, ...)
	itemTooltipDone = false
end

local function posthookscript(tooltip,script,hook)
	local orig = tooltip:GetScript(script)
	local stub = function(...)
		if orig then orig(...) end
		if hook then hook(...) end
	end
	tooltip:SetScript(script,stub)	
end

local Real_SetAuctionItem=GameTooltip.SetAuctionItem
local function Hook_SetAuctionItem(self,type,index)
	if type=="list" then
		local name, texture, count, quality, canUse, level, minBid, minIncrement, buyoutPrice, bidAmount, highestBidder, owner, sold = GetAuctionItemInfo("list", index)
		local link = GetAuctionItemLink("list", index)
		ahId = P.idOfItem(link)
		ahPrice = (buyoutPrice or bidAmount)/count
	end
	return Real_SetAuctionItem(self,type,index)
end
GameTooltip.SetAuctionItem=Hook_SetAuctionItem

function Tooltip:OnInitialize()
	posthookscript(GameTooltip,"OnTooltipSetItem",Hook_GameTooltip_OnTooltipSetItem)
	posthookscript(GameTooltip,"OnTooltipCleared",Hook_GameTooltip_OnTooltipCleared)
	
	posthookscript(ItemRefTooltip,"OnTooltipSetItem",Hook_ItemRefTooltip_OnTooltipSetItem)
	posthookscript(ItemRefTooltip,"OnTooltipCleared",Hook_ItemRefTooltip_OnTooltipCleared)
end
